//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : Cstartup_SAM7.c
//* Object              : Low level initializations written in C for IAR Tools
//* Creation            : 12/Jun/04
//* 1.2   28/Feb/05 JPP : LIB change AT91C_WDTC_WDDIS & PLL
//* 1.3   21/Mar/05 JPP : Change PLL Wait time
//*----------------------------------------------------------------------------

// Include the board file description
#include "Board.h"
#define FLASH_PAGE_SIZE_WORDS 32  //for SAM7S64
#define SECTIME 0x49200 / 100     //1msec
#define DELAYTIME 200            // Wait for 0xEA until this time in msec

// The following functions must be write in ARM mode this function called directly
// by exception vector
extern void AT91F_Spurious_handler(void);
extern void AT91F_Default_IRQ_handler(void);
extern void AT91F_Default_FIQ_handler(void);
extern void main(void);

unsigned int flash_buf[FLASH_PAGE_SIZE_WORDS];

void usart_putc(unsigned char data)
{		
	while((*AT91C_DBGU_CSR & AT91C_US_TXRDY) == 0);
	*AT91C_DBGU_THR = (data & 0xFF);
}

void flash_write(unsigned int page)
{
	unsigned int* flash;
	unsigned int i;

        *AT91C_MC_FCR = (0x5A<<24) | (page<<8&AT91C_MC_PAGEN) | 4;    //Unlock Region
        while((*AT91C_MC_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY );

	flash = (unsigned int *) 0x00100000 + 4*page*FLASH_PAGE_SIZE_WORDS;
	
	for(i=0; i < FLASH_PAGE_SIZE_WORDS; i++) {
		flash[i] = flash_buf[i];
	}
	
	*AT91C_MC_FCR = (0x5A<<24) | (page<<8) | 1;// | (1 << 8); //0x5A000001; //
	while((*AT91C_MC_FSR & AT91C_MC_FRDY) == 0);
        while((*AT91C_MC_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY );
}

int usart_getc1( void ) /* returns true if character in receive buffer */
{
	if ( *AT91C_DBGU_CSR & AT91C_US_RXRDY)
	{
          while (!(*AT91C_DBGU_CSR & AT91C_US_RXRDY));   /* Wait for Full Rx Buffer */
          return (*AT91C_DBGU_RHR);                      /* Read Character */
	}
	return 0;
}

int usart_getc( void ) /* returns true if character in receive buffer */
{
          while (!(*AT91C_DBGU_CSR & AT91C_US_RXRDY));   /* Wait for Full Rx Buffer */
          return (*AT91C_DBGU_RHR);                      /* Read Character */
}

unsigned int BaudrateCalcSt (
	const unsigned int main_clock, // \arg peripheral clock
	const unsigned int baud_rate)  // \arg UART baudrate
{
	unsigned int baud_value = (10 * main_clock/(baud_rate * 16));
	if ((baud_value % 10) >= 5)
		baud_value = (baud_value / 10) + 1;
	else
		baud_value /= 10;
	return baud_value;
}


void usartInit(void)
{
        *AT91C_PMC_PCER = 1 << AT91C_ID_SYS;
	
        *AT91C_PIOA_PDR = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Disable PIO
        *AT91C_PIOA_ASR = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Enable PeripherialA

        *AT91C_DBGU_BRGR  = BaudrateCalcSt(MCK, 115200);
        *AT91C_DBGU_MR    = AT91C_US_PAR_NONE | AT91C_US_CHMODE_NORMAL;
        *AT91C_DBGU_CR    = AT91C_US_RXEN | AT91C_US_TXEN;

        *AT91C_PITC_PIMR = 0x000fffff | AT91C_PITC_PITEN;
}

void StartBootLoader(void)
{	
unsigned int i, blpage;
unsigned char BootSt, BootExit, Needremap;
unsigned long Time;
unsigned int *memflash, *memram;

	usartInit();
        BootSt=0;
	Time = *AT91C_PITC_PIIR;
        while((*AT91C_PITC_PIIR - Time) < (DELAYTIME * SECTIME))
        {
            if((usart_getc1())==0xea)
            {
                BootSt=1;
                memflash = (unsigned int *) 0x00100000;
                memram = (unsigned int *) 0x00200000;
                Needremap=0;
                for(i=0;i<10;i++)
                {
                  if(memram[i] != memflash[i]) Needremap = 1;
                }
                if(Needremap)
                {
                  for(i=0;i<0x400;i++) memram[i] = memflash[i];
                  // Do the memory remapping
                  *AT91C_MC_RCR = 1;
                }
                usart_putc(12);
                break;
            }
        }

        if(BootSt==1)
        {
          BootExit=0;
          while (!BootExit)
  	  {	
  	        switch(usart_getc())
		{
		        case 0xeb:
		                BootExit=1;
		                break;
				
			case 0xec: //Write
				
				//Get page (2 bytes)
				blpage = usart_getc();
				blpage += usart_getc()<<8;
				
				//Get data
				for(i=0; i<FLASH_PAGE_SIZE_WORDS; i++) {
					flash_buf[i] = (usart_getc());
					flash_buf[i] += (usart_getc()<<8);
					flash_buf[i] += (usart_getc()<<16);
					flash_buf[i] += (usart_getc()<<24);
				}
				flash_write(blpage);
				//Confirm ok
				usart_putc(1);
				break;

			default:
				break;
		}
	  }
	}
	if(BootSt==1)
	{
           *AT91C_RSTC_RCR = 0xa500000f;    //Reset device
	}
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_LowLevelInit
//* \brief This function performs very low level HW initialization
//*        this function can be use a Stack, depending the compilation
//*        optimization mode
//*----------------------------------------------------------------------------
void AT91F_LowLevelInit( void) @ "ICODE"
{
int            i;
AT91PS_PMC     pPMC = AT91C_BASE_PMC;


    //* Set Flash Waite sate
	//  Single Cycle Access at Up to 30 MHz, or 40
	//  if MCK = 47923200 I have 50 Cycle for 1 usecond ( flied MC_FMR->FMCN
	    AT91C_BASE_MC->MC_FMR = ((AT91C_MC_FMCN)&(48 <<16)) | AT91C_MC_FWS_1FWS ;

    //* Watchdog Disable
        AT91C_BASE_WDTC->WDTC_WDMR= AT91C_WDTC_WDDIS;

	//* Set MCK at 47 923 200
    // 1 Enabling the Main Oscillator:
        // SCK = 1/32768 = 30.51 uSecond
    	// Start up time = 8 * 6 / SCK = 56 * 30.51 = 1,46484375 ms
       pPMC->PMC_MOR = (( AT91C_CKGR_OSCOUNT & (0x06 <<8) | AT91C_CKGR_MOSCEN ));
        // Wait the startup time
        while(!(pPMC->PMC_SR & AT91C_PMC_MOSCS));
	// 2 Checking the Main Oscillator Frequency (Optional)
	// 3 Setting PLL and divider:
		// - div by 5 Fin = 3,6864 =(18,432 / 5)
		// - Mul 25+1: Fout =	95,8464 =(3,6864 *26)
		// for 96 MHz the erroe is 0.16%
		// Field out NOT USED = 0
		// PLLCOUNT pll startup time estimate at : 0.844 ms
		// PLLCOUNT 28 = 0.000844 /(1/32768)
       pPMC->PMC_PLLR = ((AT91C_CKGR_DIV & 0x05) |
                         (AT91C_CKGR_PLLCOUNT & (28<<8)) |
                         (AT91C_CKGR_MUL & (25<<16)));

        // Wait the startup time
        while(!(pPMC->PMC_SR & AT91C_PMC_LOCK));
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));
 	// 4. Selection of Master Clock and Processor Clock
 	// select the PLL clock divided by 2
 	    pPMC->PMC_MCKR =  AT91C_PMC_PRES_CLK_2 ;
 	    while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

 	    pPMC->PMC_MCKR |= AT91C_PMC_CSS_PLL_CLK  ;
 	    while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

	// Set up the default interrupts handler vectors
	AT91C_BASE_AIC->AIC_SVR[0] = (int) AT91F_Default_FIQ_handler ;
	for (i=1;i < 31; i++)
	{
	    AT91C_BASE_AIC->AIC_SVR[i] = (int) AT91F_Default_IRQ_handler ;
	}
	AT91C_BASE_AIC->AIC_SPU  = (int) AT91F_Spurious_handler ;
	
	//Here Comes the BootLoader
	StartBootLoader();

}





