#include "include/AT91SAM7S64.h"
#include "include/regptr.h"

#include "wait.h"
#include "glcd_cv12864b.h"
#include "rs232.h"
#include "font5x7.h"

U8 GlcdBuff[1024];
//--------------------------------------------------------------------------
void LCD_strobe()
{
    pPIO->PIO_SODR = LCD_E;
    Wait(10,USEC);
    pPIO->PIO_CODR = LCD_E;
    Wait(10,USEC);
}
//--------------------------------------------------------------------------
void WriteLCD(U16 lcmd)
{
//lcmd: RS,RW,D7,D6,D5,D4,D3,D2,D1,D0
    LCD_busy();
    if((lcmd & 0x200)!=0) pPIO->PIO_SODR = LCD_RS; else pPIO->PIO_CODR = LCD_RS;
    if((lcmd & 0x100)!=0) pPIO->PIO_SODR = LCD_RW; else pPIO->PIO_CODR = LCD_RW;
    lcmd &= 0xff;
    pPIO->PIO_CODR = LCD_DATABUS;
    pPIO->PIO_SODR = LCD_DATABUS & (lcmd<< LCD_SHIFT);
    LCD_strobe();
}
//--------------------------------------------------------------------------
void LCD_Sel(U8 Csel)
{
   switch(Csel)
    {
      case RIGHT:
          pPIO->PIO_SODR = LCD_CSA;
          pPIO->PIO_CODR = LCD_CSB;
          break;

      case LEFT:
          pPIO->PIO_CODR = LCD_CSA;
          pPIO->PIO_SODR = LCD_CSB;
          break;

      case ALL:
          pPIO->PIO_CODR = LCD_CSA;
          pPIO->PIO_CODR = LCD_CSB;
          break;

      case NONE:
          pPIO->PIO_SODR = LCD_CSA | LCD_CSB;
          break;


      default:
          pPIO->PIO_SODR = LCD_CSA;
          pPIO->PIO_CODR = LCD_CSB;
          break;
    }
}
//--------------------------------------------------------------------------
void LCD_busy(void)
{
U32 busy;
	
    busy=1;
    pPIO->PIO_ODR = LCD_DATABUS;
    pPIO->PIO_SODR = LCD_RW;
    pPIO->PIO_CODR = LCD_RS;
    while(busy!=0)
    {
      pPIO->PIO_SODR = LCD_E;
      busy = (pPIO->PIO_PDSR & LCD_DATABUS) & (0x80<<LCD_SHIFT); //check busy flag
      //busy = (pPIO->PIO_PDSR & LCD_D7);
      //SendRS232_Byte(pPIO->PIO_PDSR & LCD_DATABUS);	
      pPIO->PIO_CODR = LCD_E;
    }
    pPIO->PIO_CODR = LCD_RW;
    pPIO->PIO_OER = LCD_DATABUS;
}
//--------------------------------------------------------------------------
void GLCD_Clear(void)
{
U8 x,y;

    LCD_Sel(ALL);
    WriteLCD(SETPAGE|0);
    WriteLCD(SETY|0);
    for(x=8; x!=0; x--)
    {
      WriteLCD(SETPAGE|x);
      WriteLCD(SETY|0);
      for(y=64; y!=0; y--)
      {
         WriteLCD(WRITE|0x00);
      }
    }
    LCD_Sel(NONE);
}
//--------------------------------------------------------------------------
void Init_GLCD(void)
{
    pPIO->PIO_PDR = LCD_DATABUS | LCD_RW | LCD_RS | LCD_E | LCD_CSA | LCD_CSB;
    pPIO->PIO_PER = LCD_DATABUS | LCD_RW | LCD_RS | LCD_E | LCD_CSA | LCD_CSB;
    pPIO->PIO_OER = LCD_DATABUS | LCD_RW | LCD_RS | LCD_E | LCD_CSA | LCD_CSB;
    pPIO->PIO_CODR = LCD_DATABUS | LCD_RW | LCD_RS | LCD_E;
    pPIO->PIO_SODR = LCD_CSA | LCD_CSB;
    Wait(200,MSEC);
    
    LCD_Sel(LEFT);
    WriteLCD(DISPOFF);
    WriteLCD(DISPSTART|0);
    WriteLCD(DISPON);

    LCD_Sel(RIGHT);
    WriteLCD(DISPOFF);
    WriteLCD(DISPSTART|0);
    WriteLCD(DISPON);
}
//--------------------------------------------------------------------------
/*
void SetPixel(U8 x, U8 y)
{
U8 temppix;

    if(x<64) LCD_Sel(LEFT); else LCD_Sel(RIGHT);
    WriteLCD(SETPAGE|(y >> 3));
    if(x>63) x -= 64;
    WriteLCD(SETY|x);
    y &= 7;
    WriteLCD(WRITE|(1<<y));
}
//--------------------------------------------------------------------------
void ClearPixel(U8 x, U8 y)
{
    if(x<64) LCD_Sel(LEFT); else LCD_Sel(RIGHT);
    WriteLCD(SETPAGE|(y >> 3));
    if(x>63) x -= 64;
    WriteLCD(SETY|x);
    y &= 7;
    WriteLCD(WRITE|(0<<y));
}
*/
//--------------------------------------------------------------------------
void SetPixel(U8 x, U8 y)
{
U8 Poffset;
U16 Paddr;

    if((x<128) & (y<64))
    {
      Poffset = (y & 7);
      Paddr = ((y & 0xf8) << 4) + x;
      GlcdBuff[Paddr] |= (1 << Poffset);
    }
}
//--------------------------------------------------------------------------
void ClearPixel(U8 x, U8 y)
{
U8 Poffset;
U16 Paddr;

    Poffset = (y & 7);
    Paddr = ((y & 0xf8) << 4) + x;
    GlcdBuff[Paddr] &= (~(1 << Poffset));
}
//--------------------------------------------------------------------------
/*
void FlushScreen(void)
{
U8 c1,c2,cx;

    for(c1=0;c1<8;c1++)
    {
      for(c2=0;c2<128;c2++)
      {
          if(c2==0) LCD_Sel(LEFT);
          if(c2==64) LCD_Sel(RIGHT);
          WriteLCD(SETPAGE|c1);
          if(c2<64) cx=c2; else cx = c2-64;
          WriteLCD(SETY|cx);
          WriteLCD(WRITE|GlcdBuff[128*c1+c2]);
      }
    }
}
*/
//--------------------------------------------------------------------------
void FlushScreen(void)
{
U8 c1,c2;

    for(c1=0;c1<8;c1++)
    {
      for(c2=0;c2<128;c2++)
      {
        if(c2==0) LCD_Sel(LEFT);
        if(c2==64) LCD_Sel(RIGHT);
        if((c2==0)|(c2==64))
        {
          WriteLCD(SETPAGE|c1);
          WriteLCD(SETY|0);
        }
        WriteLCD(WRITE|GlcdBuff[128*c1+c2]);
      }
    }
}
//--------------------------------------------------------------------------
void GLCD_Text(U8 x, U8 y, U8 *szoveg,U8 Inv)
{
U8 yc;

   while(*szoveg)
   {
      GLCD_Char(x,y,*szoveg,Inv);
      if(Inv)
      {
        for(yc=0;yc<8;yc++) SetPixel(x+5,y+yc);
      }
      x=x+6;
      *szoveg++;
   }
}
//--------------------------------------------------------------------------
void GLCD_Char(U8 x, U8 y, U8 chr,U8 Inv)
{
U8 yb,ch,xcnt;

   for(xcnt=0;xcnt<5;xcnt++)
   {
     ch=Font5x7[5*(chr-0x20)+xcnt];
     for(yb=0;yb<8;yb++)
     {
       if(Inv)
       {
         if(ch & (1<<yb)) ClearPixel(x+xcnt,y+yb); else SetPixel(x+xcnt,y+yb);
       }
       else
       {
         if(ch & (1<<yb)) SetPixel(x+xcnt,y+yb);       
       }
     }
   }
}
//--------------------------------------------------------------------------
void GLCD_Rec(U8 x1, U8 y1, U8 x2, U8 y2)
{
U8 xc,yc;

    for(xc=x1;xc<(x2+1);xc++)
    {
      SetPixel(xc,y1);
      SetPixel(xc,y2);
    }
    for(yc=y1;yc<(y2+1);yc++)
    {
      SetPixel(x1,yc);
      SetPixel(x2,yc);
    }
}
//--------------------------------------------------------------------------
