/*
**********************************************************************************************
*
*   SAM7S64 BootLoader
*
*   Created:  06/05/2006
*   Author:   Attila Meszaros
*
**********************************************************************************************
*/
#include "include/at91sam7s64.h"

#define MCK 47923200
#define FLASH_PAGE_SIZE_WORDS 32  //for SAM7S64
#define SECTIME 0x49200 / 100     //1msec
#define DELAYTIME 200            // Wait for 0xEA until this time in msec

unsigned int flash_buf[FLASH_PAGE_SIZE_WORDS];

void flash_init (void)
{
    *AT91C_MC_FMR = ((AT91C_MC_FMCN)&(72 <<16)) | AT91C_MC_FWS_1FWS;
}

void usart_putc(unsigned char data)
{		
	while((*AT91C_DBGU_CSR & AT91C_US_TXRDY) == 0);
	*AT91C_DBGU_THR = (data & 0xFF);
}

__ramfunc void flash_write(unsigned int page)
{
	unsigned int* flash;
	unsigned int i;

        if((page>31)&(page<512))
        {
        *AT91C_MC_FCR = (0x5A<<24) | (page<<8&AT91C_MC_PAGEN) | 4;    //Unlock Region
        while((*AT91C_MC_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY );

	//flash = (unsigned int *) 0x00100000 + 4*page*FLASH_PAGE_SIZE_WORDS;
	flash = (unsigned int *) 0x00100000 + 4*page*FLASH_PAGE_SIZE_WORDS;
	
	for(i=0; i < FLASH_PAGE_SIZE_WORDS; i++) {
		flash[i] = flash_buf[i];
	}
	
	*AT91C_MC_FCR = (0x5A<<24) | (page<<8) | 1;// | (1 << 8); //0x5A000001; //
	while((*AT91C_MC_FSR & AT91C_MC_FRDY) == 0);
        while((*AT91C_MC_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY );
        }
}

int usart_getc1( void ) /* returns true if character in receive buffer */
{
	if ( *AT91C_DBGU_CSR & AT91C_US_RXRDY)
	{
          while (!(*AT91C_DBGU_CSR & AT91C_US_RXRDY));   /* Wait for Full Rx Buffer */
          return (*AT91C_DBGU_RHR);                      /* Read Character */
	} else return(0);
}

int usart_getc( void ) /* returns true if character in receive buffer */
{
          while (!(*AT91C_DBGU_CSR & AT91C_US_RXRDY));   /* Wait for Full Rx Buffer */
          return (*AT91C_DBGU_RHR);                      /* Read Character */
}

unsigned int BaudrateCalcSt (
	const unsigned int main_clock, // \arg peripheral clock
	const unsigned int baud_rate)  // \arg UART baudrate
{
	unsigned int baud_value = (10 * main_clock/(baud_rate * 16));
	if ((baud_value % 10) >= 5)
		baud_value = (baud_value / 10) + 1;
	else
		baud_value /= 10;
	return baud_value;
}


void usartInit(void)
{
        *AT91C_PMC_PCER = 1 << AT91C_ID_SYS;
	
        *AT91C_PIOA_PDR = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Disable PIO
        *AT91C_PIOA_ASR = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Enable PeripherialA

        *AT91C_DBGU_BRGR  = BaudrateCalcSt(MCK, 115200);
        *AT91C_DBGU_MR    = AT91C_US_PAR_NONE | AT91C_US_CHMODE_NORMAL;
        *AT91C_DBGU_CR    = AT91C_US_RXEN | AT91C_US_TXEN;

        *AT91C_PITC_PIMR = 0x000fffff | AT91C_PITC_PITEN;
}

void main(void)
{	
unsigned char BootSt, BootExit;
unsigned long Time;
unsigned int i, blpage;

        usartInit();
	flash_init();
        BootSt=0;
	Time = *AT91C_PITC_PIIR;
        while((*AT91C_PITC_PIIR - Time) < (DELAYTIME * SECTIME))
        {
            if((usart_getc1())==0xea)
            {
                BootSt=1;
                usart_putc(12);
                break;
            }
        }
        if(BootSt==1)
        {
          BootExit=0;
          while (!BootExit)
  	  {	
  	        switch(usart_getc())
		{
		        case 0xeb:
		                BootExit=1;
		                break;
				
			case 0xec: //Write
				//Get page (2 bytes)
				blpage = usart_getc();
				blpage += usart_getc()<<8;
				
				//Get data
				for(i=0; i<FLASH_PAGE_SIZE_WORDS; i++) {
					flash_buf[i] = (usart_getc());
					flash_buf[i] += (usart_getc()<<8);
					flash_buf[i] += (usart_getc()<<16);
					flash_buf[i] += (usart_getc()<<24);
				}
				flash_write(blpage);
				//Confirm ok
				usart_putc(1);
				break;

			default:
				break;
		}
	  }
	}
	if(BootSt==1)
	{
           *AT91C_RSTC_RCR = 0xa500000f;    //Reset device
	}
}
